// UI elements.
const deviceNameLabel = document.getElementById('device-name');
const connectButton = document.getElementById('connect');
const disconnectButton = document.getElementById('disconnect');
const terminalContainer = document.getElementById('terminal');
const sendForm = document.getElementById('send-form');
const inputField = document.getElementById('input');

// Helpers.
const defaultDeviceName = 'Terminal';
const terminalAutoScrollingLimit = terminalContainer.offsetHeight / 2;
let isTerminalAutoScrolling = true;

const scrollElement = (element) => {
  const scrollTop = element.scrollHeight - element.offsetHeight;

  if (scrollTop > 0) {
    element.scrollTop = scrollTop;
  }
};

const logToTerminal = (message, type = '') => {
  terminalContainer.insertAdjacentHTML('beforeend',
      `<div${type && ` class="${type}"`}>${message}</div>`);

  if (isTerminalAutoScrolling) {
    scrollElement(terminalContainer);
  }
};

const logToTerminalBin = (message, type = '') => {
  let stringElement = document.getElementById(type);
  stringElement.innerHTML = message
};



// Obtain configured instance.
const terminal = new BluetoothTerminal();

    
twoByteToInt = function (data) {
  if (data > 32767) {
  return (data - 65536);
  } else {
  return data  
  }
}

function convertToHex(str) {
  var hex = '';
  for(var i=0;i<str.length;i++) {
      hex += ''+str.charCodeAt(i).toString(16);
  }
  return hex;
}

arToHex = function (arrByte) {
  let strHex = ""
     for (let index = 0; index < arrByte.length; index++) {
         strHex += arrByte[index].toString(10) +"," ;
      
    }
  return strHex 
  }


// Override `receive` method to log incoming data to the terminal.
terminal.receive = function(data) {
  
  logToTerminal(data, 'in');
  // console.log(convertToHex(data));
  // console.log(typeof data );
};

let dist=0
let energy=0
terminal.receiveBin = function(data) {
  let inId = ""
  let dataBin = ""
  
  let current = twoByteToInt(data[10] * 256 + data[11])/100
  dataBin = ("Cur = " + current.toFixed(2) + "A; ")
  logToTerminalBin(dataBin,"stroke1el1");
  
  let voltage = twoByteToInt(data[2] * 256 + data[3])/100
  dataBin = ("V = " + voltage.toFixed(2) + "v; ")
  logToTerminalBin(dataBin,"stroke1el2");
  
  let speed = twoByteToInt(data[4] * 256 + data[5])/100
  dataBin = ("Sped = " + speed.toFixed(2) + "km/h; ")
  logToTerminalBin(dataBin,"stroke1el3");
   
  dist += speed
  dataBin = ("Dist = " + (dist/10000).toFixed(3) + "km; ")
  logToTerminalBin(dataBin,"stroke1el4");
  
  let temperature = twoByteToInt(data[12] * 256 + data[13])/100
  dataBin = ("t°C = " + temperature.toFixed(2) + "°C; ")
  logToTerminalBin(dataBin,"stroke1el5");
  
  let power = voltage * current
  dataBin = ("Power = " + (power/1000).toFixed(3) +"kWt; ")
  logToTerminalBin(dataBin,"stroke2el1");
  
  energy += power
  dataBin = ("Energy = " + (energy/10000000).toFixed(3) + "kW/h; ")
  logToTerminalBin(dataBin,"stroke2el2");
  
  efficiency = power / speed
  dataBin = ("Eff..y = " + (efficiency).toFixed(3) + "Wh/km; ")
  logToTerminalBin(dataBin,"stroke2el3");
  
};
// Override default log method to output messages to the terminal and console.
terminal._log = function(...messages) {
  // We can't use `super._log()` here.
  messages.forEach((message) => {
    logToTerminal(message);
    console.log(message); // eslint-disable-line no-console
  });
};

// Implement own send function to log outcoming data to the terminal.
const send = (data) => {
  terminal.send(data).
      then(() => logToTerminal(data, 'out')).
      catch((error) => logToTerminal(error));
};

// Bind event listeners to the UI elements.
connectButton.addEventListener('click', () => {
  terminal.connect().
      then(() => {
        deviceNameLabel.textContent = terminal.getDeviceName() ?
            terminal.getDeviceName() : defaultDeviceName;
      });
});

disconnectButton.addEventListener('click', () => {
  terminal.disconnect();
  deviceNameLabel.textContent = defaultDeviceName;
});

sendForm.addEventListener('submit', (event) => {
  event.preventDefault();

  send(inputField.value);

  inputField.value = '';
  inputField.focus();
});

// Switch terminal auto scrolling if it scrolls out of bottom.
terminalContainer.addEventListener('scroll', () => {
  const scrollTopOffset = terminalContainer.scrollHeight -
      terminalContainer.offsetHeight - terminalAutoScrollingLimit;

  isTerminalAutoScrolling = (scrollTopOffset < terminalContainer.scrollTop);
});
